require "spec_helper"
require "rack"

describe SocialAvatarProxy::App do
  subject do
    Rack::MockRequest.new(app)
  end
  
  let(:app) { SocialAvatarProxy::App.new }
  
  let(:successful_response) do
    response = Net::HTTPSuccess.new("1.1", 200, "Success")
    response.stub(:stream_check).and_return(true)
    response.stub(:read_body).and_return("data")
    response
  end
  
  let(:not_found_response) do
    Net::HTTPNotFound.new("1.1", 404, "Not Found")
  end
  
  let(:response) { nil }
  
  let(:image_path) do
    File.join(File.dirname(__FILE__), "../fixtures/image.jpg")
  end
  
  before(:each) do
    SocialAvatarProxy.send(:remove_const, :Config)
    SocialAvatarProxy.const_set(:Config, SocialAvatarProxy::Configuration.new)
    klass = SocialAvatarProxy::Avatar
    klass.any_instance.stub(:response).and_return(response)
  end
  
  describe "::call" do
    let(:response) { successful_response }
    let(:app) { SocialAvatarProxy::App }

    it "should be callable from the class" do
      expect(subject.get("/twitter/dave").status).to eq(200)
    end
  end
  
  describe "::routes" do
    subject { SocialAvatarProxy::App }
    
    it "should return a object providing the path helpers" do
      expect(subject.routes.facebook_avatar_path("ryandtownsend")).to match(/^\/.*$/)
      expect(subject.routes.twitter_avatar_path(2202971)).to match(/^\/.*$/)
    end
  end
  
  describe "#routes" do
    subject { app }

    it "should return a object providing the path helpers" do
      expect(subject.routes.facebook_avatar_path("ryandtownsend")).to match(/^\/.*$/)
      expect(subject.routes.twitter_avatar_path(2202971)).to match(/^\/.*$/)
    end
  end
  
  context "given an invalid path" do
    let(:path) { "/unknown" }

    it "should return a 404 response" do
      expect(subject.get(path).status).to eq(404)
    end
  end
  
  context "given a valid path" do
    context "when the remote server times out" do
      before(:each) do
        app.should_receive(:response).and_raise(SocialAvatarProxy::TimeoutError)
      end
      
      it "should return a 504 response" do
        expect(subject.get("/facebook/61413673").status).to eq(504)
      end
    end
    
    context "when the remote server has too many redirects" do
      before(:each) do
        app.should_receive(:response).and_raise(SocialAvatarProxy::TooManyRedirectsError)
      end
      
      it "should return a 502 response" do
        expect(subject.get("/facebook/61413673").status).to eq(502)
      end
    end
    
    context "that finds an existing avatar" do
      let(:response) { successful_response }
      let(:path) { "/facebook/61413673" }
      
      it "should return a 200 response" do
        expect(subject.get(path).status).to eq(200)
      end
      
      context "with memcache in place" do
        before(:each) do
          SocialAvatarProxy.configure do
            memcache do
              enable
            end
          end
        end
        
        it "should attempt to get the image from the cache" do
          expect(subject.get(path).status).to eq(200)
        end
      end
      
      context "with file cache in place" do
        before(:each) do
          SocialAvatarProxy.configure do
            file_cache do
              directory File.join(File.dirname(__FILE__), "../../.tmp")
            end
          end
        end
        
        it "should attempt to get the image from the cache" do
          expect(subject.get(path).status).to eq(200)
        end
      end
      
      context "with memcache and the file cache in place" do
        before(:each) do
          SocialAvatarProxy.configure do
            memcache do
              enable
            end
            file_cache do
              directory File.join(File.dirname(__FILE__), "../../.tmp")
            end
          end
        end
        
        it "should attempt to get the image from the cache" do
          expect(subject.get(path).status).to eq(200)
        end
      end
      
      context "with Cache-Control set with Max-Age and Max-Stale as 1 day" do
        before(:each) do
          SocialAvatarProxy.configure do
            http_cache do
              cache_control({
                max_stale: 86400,
                max_age: 86400,
                public: true
              })
            end
          end
        end
        
        it "should return a Cache-Control header" do
          expect(subject.get(path).headers["Cache-Control"]).to eq("max-stale=86400, max-age=86400, public")
        end
      end
      
      context "given Cache-Control is turned off" do
        before(:each) do
          SocialAvatarProxy.configure do
            http_cache do
              disable
            end
          end
        end
        
        it "should not return a Cache-Control header" do
          expect(subject.get(path).headers["Cache-Control"]).to be_nil
        end
      end
      
      context "given Cache-Control is set to private" do
        before(:each) do
          SocialAvatarProxy.configure do
            http_cache do
              cache_control({ public: false })
            end
          end
        end
        
        it "should return a Cache-Control header marked for private use" do
          expect(subject.get(path).headers["Cache-Control"]).to match(/private$/)
        end
      end
      
      context "given Expires is turned off" do
        before(:each) do
          SocialAvatarProxy.configure do
            http_cache do
              cache_control({ expires: false })
            end
          end
        end
        
        it "should not return a Cache-Control header" do
          expect(subject.get(path).headers["Expires"]).to be_nil
        end
      end
      
      context "given Expires is set to two days" do
        before(:each) do
          SocialAvatarProxy.configure do
            http_cache do
              expires 86400 * 2
            end
          end
        end
        
        it "should not return a Cache-Control header" do
          expect(subject.get(path).headers["Expires"]).to eq((Time.now + (86400 * 2)).httpdate)
        end
      end
    end
    
    context "that fails to find an avatar" do
      let(:response) { not_found_response }
      let(:path) { "/facebook/someRandomUserThatDoesntExist" }

      it "should return a 404 response" do
        expect(subject.get(path).status).to eq(404)
      end
      
      context "with a custom image" do
        before(:each) do
          ip = image_path
          SocialAvatarProxy.configure do
            default_image(ip)
          end
        end
        
        it "should render the image" do
          expect(subject.get(path)["Content-Type"]).to eq("image/jpeg")
        end
      end
    end
  end
end